#define F_CPU 1000000
#include <avr/io.h>
#include <util/delay.h>
#include "ps2.h"

#define PS2_DAT PC5
#define PS2_CLK PC4

/*
Connections of the little ps2-to-breadboard thingy, viewed from the side you
plug in the mouse:

  ------- 
 |  ___  |
 | /   \ |
 | \___/ |
 |       |
-----------
| | | | | |
1 2 3 4 5 6

1 - ground
2 - data
3 - clock
6 - +5v

How to use these files:
- Copy ps2.h and ps2.c into the folder containing the project you want to extend
- In the Makefile, add 'ps2.c' to the SRC-line
(e.g. make it into SRC=main.c ps2.c)
- In main.c, add an #include "ps2.h" at the top of the file
- Now you can call the routines defined in ps2.h.
*/


//This routine initializes the signal levels on the port. Call before calling
//the other routines!
void ps2Init() {
	//Both clock and data should have their pull-ups enabled but
	//should not be outputs
	PORTC|=(1<<PS2_DAT)|(1<<PS2_CLK);
	DDRC&=~((1<<PS2_DAT)|(1<<PS2_CLK));
	_delay_ms(1000);
}

//Internal: wait for a clock edge and read a bit from the data pin
char ps2GetBit() {
	char ret;
	while(PINC&(1<<PS2_CLK)); //wait for low clock line
	ret=(PINC&(1<<PS2_DAT));
	while(!(PINC&(1<<PS2_CLK))); //wait for high clock line
	return ret;
}

//Internal: write a bit to the data pin and wait for the clock pin
void ps2SendBit(char bit) {
	if (bit) {
		//High: only pull-up
		PORTC|=(1<<PS2_DAT);
		DDRC&=~(1<<PS2_DAT);
	} else {
		//Low: Actively pull low
		PORTC&=~(1<<PS2_DAT);
		DDRC|=(1<<PS2_DAT);
	}
	while(!(PINC&(1<<PS2_CLK))); //wait for high clock line
	while(PINC&(1<<PS2_CLK)); //wait for low clock line
}

//Receive a byte from the PS2-port
unsigned char ps2Recv() {
	unsigned char x,ret=0;
	//Make clock high so we can receive data
	DDRC&=~(1<<PS2_CLK);
	PORTC|=(1<<PS2_CLK);
	//Wait for clk line to stabilise. The PS2-device has to wait 50uS anyway.
	_delay_us(30); 
	//Wait for low start bit
	ps2GetBit() ;
	//Get 8 data bits
	for (x=0; x<8; x++) {
		ret>>=1;
		if (ps2GetBit()) ret|=0x80;
	}
	//Get parity bit
	ps2GetBit();
	//Get stop bit
	ps2GetBit();
	//Bring the clock line low so we won't get data while we're doing other things
	PORTC&=~(1<<PS2_CLK); //low output
	DDRC|=(1<<PS2_CLK); //make the line an output
	return ret;
}

//Send a byte to the ps2-port
void  ps2Send(unsigned char c) {
	char x, onecount;
	//Bring the clock line low
	PORTC&=~(1<<PS2_CLK); //low output
	DDRC|=(1<<PS2_CLK); //make the line an output
	//Protocol: wait 100uS at least with the clock pulled low
	_delay_us(150);
	//Bring the data line low
	PORTC&=~(1<<PS2_DAT); //Low output
	DDRC|=(1<<PS2_DAT); //make the line an output
	//Release the clock line
	DDRC&=~(1<<PS2_CLK); //line is input
	PORTC|=(1<<PS2_CLK); //enable pull-up
	_delay_us(30);
	//Wait for the device to bring the clock line low
	while(PINC&(1<<PS2_CLK));
	//Send the 8 bits
	onecount=0;
	for (x=0; x<8; x++) {
		ps2SendBit(c&1);
		if (c&1) onecount++;
		c>>=1;
	}
	//Send parity bit
	ps2SendBit(!(onecount&1));
	//Send stop bit
	ps2SendBit(1);
	//Get ack bit (actually, ignore it)
	ps2GetBit();
	//Bring the clock line low so we won't get data while we're doing other things
	PORTC&=~(1<<PS2_CLK); //low output
	DDRC|=(1<<PS2_CLK); //make the line an output
}
